#include "AStarFibHeap.h"

/*
*-------------------------------------------------------------------------------------------------------
*/
AStarFibHeap::AStarFibHeap() {

	// Create the heap
	heap = fibheap_new();
}

AStarFibHeap::~AStarFibHeap() {
	
	// Delete the heap
	fibheap_delete(heap);
}

/*
*-------------------------------------------------------------------------------------------------------
*/
void AStarFibHeap::insert(AStarNode* _node, long _heuristic) {

	// Insert the node and remember the fib_node object
	fibnode_t fibNode = fibheap_insert(heap, _heuristic, _node);
	_node->fibHeapNode = fibNode;
}

/*
*-------------------------------------------------------------------------------------------------------
*/
void AStarFibHeap::erase(AStarNode* _node) {

	// Given node MUST HAVE already been added to the fibonacci heap
	fibheap_delete_node(heap, _node->fibHeapNode);
}

/*
*-------------------------------------------------------------------------------------------------------
*/
bool AStarFibHeap::empty() {
	return fibheap_empty(heap);
}

/*
*-------------------------------------------------------------------------------------------------------
*/
void AStarFibHeap::clear() {
	// Delete the heap
	fibheap_delete(heap);
	
	// Create a new heap
	heap = fibheap_new();
}

/*
*-------------------------------------------------------------------------------------------------------
*/
void AStarFibHeap::setHeuristic(AStarNode* _node, long _heuristic) {

	// Check if list is empty
	if(fibheap_empty(heap)) {
		return;
	}
	
	// Given node MUST HAVE already been added to the fibonacci heap
	fibheap_delete_node(heap, _node->fibHeapNode);
	
	// Reinsert the node with the changed heuristic
	insert(_node, _heuristic);
}

/*
*-------------------------------------------------------------------------------------------------------
*/
AStarNode* AStarFibHeap::getBestHeuristicNode() {
	
	// Check if list is empty
	if(fibheap_empty(heap)) {
		return NULL;
	}
	
	// Return the minimum node
	return (AStarNode*)(fibheap_min(heap));
}
