#include "FindParkingLot.h"
#include "Robot.h"

#define DEBUGOUTPUT
#undef DEBUGOUTPUT

#include <cv.h>
#ifdef DEBUGOUTPUT
#include <highgui.h>
#endif


bool findParkingLot( double*  laserScanPoints, 
					size_t laserScanLength,
					Position* pOdoMetrie,
					Point &leftBorderPoint, 
					Point &leftTopPoint, 
					Point &leftBottomPoint,
					Point &rightBottomPoint,
					Point &rightTopPoint, 
					Point &rightBorderPoint )
					
{

	if( laserScanPoints == NULL || pOdoMetrie == NULL )
		return false;
	double maxX = 0.;
	double maxY = 0.;
	double minX = 0.;
	double minY = 0.;
	double odoMetrieX = -pOdoMetrie->getX();
	double odoMetrieY = -pOdoMetrie->getY();
	double odoMetrieTheta = pOdoMetrie->getTheta();
	bool notFirst = false;
	std::vector< Point > pointsVector(laserScanLength);
	size_t pointsIndex = 0;
	for( unsigned int i = 0; i < laserScanLength; ++i )
	{
		pointsVector[pointsIndex].x = laserScanPoints[i] * cos( (pointsIndex/(double)laserScanLength)*M_PI /*+ odoMetrieTheta*/ ) ;//+ odoMetrieX;
		pointsVector[pointsIndex].y = laserScanPoints[i] * sin( (pointsIndex/(double)laserScanLength)*M_PI /*+ odoMetrieTheta*/ ) ;//+ odoMetrieY;
		if( notFirst ) 
		{
			maxX = (maxX<pointsVector[pointsIndex].x) ? pointsVector[pointsIndex].x : maxX;
			maxY = (maxY<pointsVector[pointsIndex].y) ? pointsVector[pointsIndex].y : maxY;
			minX = (minX>pointsVector[pointsIndex].x) ? pointsVector[pointsIndex].x : minX;
			minY = (minY>pointsVector[pointsIndex].y) ? pointsVector[pointsIndex].y : minY;
		}
		else
		{
			maxX = pointsVector[pointsIndex].x;
			maxY = pointsVector[pointsIndex].y;
			minX = pointsVector[pointsIndex].x;
			minY = pointsVector[pointsIndex].y;
			notFirst = true;
		}
		pointsIndex++;
	}
	
// 	Point robotPoint;
// 	robotPoint.x = odoMetrieX;
// 	robotPoint.y = odoMetrieY;
// 	pointsVector.push_back( robotPoint );

	CvSize size;

	size.height = IMAGESIZEY;
	size.width = IMAGESIZEX;
	
	double factorX = IMAGESIZEX/(maxX - minX);
	double shiftX = minX * factorX;
	double factorY = IMAGESIZEY/(maxY - minY);
	double shiftY = minY * factorY;
	IplImage* bild = cvCreateImage( size, 8, 1 );
	cvSetZero( bild );
	for( std::vector< Point >::iterator pointsIt = pointsVector.begin();
		pointsIt != pointsVector.end();
		++pointsIt )
	{
		int indexX = static_cast<int>(pointsIt->x * factorX);
		int indexY = static_cast<int>(pointsIt->y * factorY);
		CvPoint point;
		point.x = static_cast<int>(indexX - shiftX);
		point.y = static_cast<int>(indexY - shiftY);
		cvLine( bild, point, point, CV_RGB( 255, 255, 255 ), 1 );
	}
	double odoMetrieXScaled = /*(odoMetrieX * factorX)*/ - shiftX;
	double odoMetrieYScaled = /*(odoMetrieY * factorY)*/ - shiftY;

#ifdef DEBUGOUTPUT
	cvNamedWindow( "Quelle", 1 );
	cvShowImage( "Quelle", bild );
#endif

	//cvDilate( bild, bild, NULL, 1 );
	//cvErode( bild, bild, NULL, 1 );

#ifdef DEBUGOUTPUT
	cvNamedWindow( "Source", 1 );
	cvShowImage( "Source", bild );
#endif 

	IplImage* color_dst = cvCreateImage( cvGetSize(bild), 8, 3 );
	cvCvtColor( bild, color_dst, CV_GRAY2BGR );
	CvMemStorage* storage = cvCreateMemStorage(0);
	CvSeq* lines = cvHoughLines2( bild, storage, CV_HOUGH_STANDARD, 1, 7*CV_PI/180, 15, 0, 0 );
#ifdef DEBUGOUTPUT
	for( int i = 0; i < lines->total && i < 20; i++ )
	{
		float* line = (float*)cvGetSeqElem(lines,i);
		float rho = line[0];
		float theta = line[1];
		CvPoint pt1, pt2;
		double a = cos(theta), b = sin(theta);
		double x0 = a*rho, y0 = b*rho;
		pt1.x = cvRound(x0 + 1000*(-b));
		pt1.y = cvRound(y0 + 1000*(a));
		pt2.x = cvRound(x0 - 1000*(-b));
		pt2.y = cvRound(y0 - 1000*(a));
		cvLine( color_dst, pt1, pt2, CV_RGB(255,0,0), 1, 8 );
	}

	cvNamedWindow( "Hough", 1 );
	cvShowImage( "Hough", color_dst );
#endif


#ifdef DEBUGOUTPUT
	IplImage* umgebung_dst = cvCreateImage( cvGetSize(bild), 8, 3 );
	cvCvtColor( bild, umgebung_dst, CV_GRAY2BGR );
#endif
	bool found = false;

	double minimumPositiveDistance = 0.f;
	int minimumPositiveIndex = -1;
	double minimumNegativeDistance = 0.f;
	int minimumNegativeIndex = -1;
	for( int i = 0; i < lines->total; ++i )
	{
		float* line = (float*)cvGetSeqElem(lines,i);
		float rho = line[0];
		float theta = line[1];

		// is the line nearley perpendicular to the robot 
		// theta must be between 60 and 120
		if( fabs(theta) < 1./3.*CV_PI || (fabs(theta) > 2./3.*CV_PI && fabs(theta) < 4./3.*CV_PI) )
		{
			// calculate the distance from the robot 
			double d = cos(theta) * (odoMetrieXScaled - rho*cos(theta) )
				+ sin(theta) * (odoMetrieYScaled - rho*sin(theta ) );
			if(fabs(theta) > 2./3.*CV_PI && fabs(theta) < 4./3.*CV_PI) 
			{
				d *= -1;
			}
			if( d > 0 && ( minimumPositiveDistance > d || minimumPositiveIndex  == -1 ) )
			{
				minimumPositiveDistance = d;
				minimumPositiveIndex = i;
			}
			if( d < 0 && ( fabs(minimumNegativeDistance) > fabs(d) || minimumNegativeIndex  == -1 ) )
			{
				minimumNegativeDistance = d;
				minimumNegativeIndex = i;
			}
		}
	}

	// test if found lines are from a parking lot 
	// we expect the robot in the middle besides the lot 
	// so the positive and negative distance must be nearly the same
	// we calculate the average distance 
	double averageDist = (fabs( minimumPositiveDistance ) + fabs( minimumNegativeDistance )) / 2.f;
	// then look if we have nearly the same distances
	if( fabs( (minimumPositiveDistance / averageDist) + ( minimumNegativeDistance / averageDist ) ) < 1e0 && averageDist < 50.f )
	{
#ifdef DEBUGOUTPUT
		if( minimumPositiveIndex != -1 )
		{
			float* line = (float*)cvGetSeqElem(lines,minimumPositiveIndex);
			float rho = line[0];
			float theta = line[1];
			CvPoint pt1, pt2;
			double a = cos(theta), b = sin(theta);
			double x0 = a*rho, y0 = b*rho;
			pt1.x = cvRound(x0 + 1000*(-b));
			pt1.y = cvRound(y0 + 1000*(a));
			pt2.x = cvRound(x0 - 1000*(-b));
			pt2.y = cvRound(y0 - 1000*(a));
			cvLine( umgebung_dst, pt1, pt2, CV_RGB(255,255,0), 1, 8 );
		}

		if( minimumNegativeIndex != -1 )
		{
			float* line = (float*)cvGetSeqElem(lines,minimumNegativeIndex);
			float rho = line[0];
			float theta = line[1];
			CvPoint pt1, pt2;
			double a = cos(theta), b = sin(theta);
			double x0 = a*rho, y0 = b*rho;
			pt1.x = cvRound(x0 + 1000*(-b));
			pt1.y = cvRound(y0 + 1000*(a));
			pt2.x = cvRound(x0 - 1000*(-b));
			pt2.y = cvRound(y0 - 1000*(a));
			cvLine( umgebung_dst, pt1, pt2, CV_RGB(0,255,255), 1, 8 );
		}
#endif
	}
	else
	{
		minimumNegativeIndex = -1;
		minimumPositiveIndex = -1;
	}

	double minimumDistance = 0.f;
	int minimumIndex = -1;
	double maximumDistance = 0.f;
	int maximumIndex = -1;
	found = false;
	for( int i = 0; i < lines->total; ++i )
	{
		float* line = (float*)cvGetSeqElem(lines,i);
		float rho = line[0];
		float theta = line[1];
		// is the line nearley perpendicular to the robot 
		// theta must be between 60 and 120
		if( theta > (1./3.)*CV_PI && theta < (2./3.)*CV_PI )
		{

			// calculate the distance from the robot 
			double d = cos(theta) * (odoMetrieXScaled - rho*cos(theta) )
				+ sin(theta) * (odoMetrieYScaled - rho*sin(theta ) );
			if( fabs(minimumDistance) > fabs(d) || minimumIndex == -1 )
			{
				minimumDistance = d;
				minimumIndex = i;
			}
			if( fabs(maximumDistance) < fabs(d) || maximumIndex == -1 )
			{
				maximumDistance = d;
				maximumIndex = i;
			}
		}
	}

#ifdef DEBUGOUTPUT
	IplImage* umgebungsModel_dst = cvCreateImage( cvGetSize(bild), 8, 3 );
	cvCvtColor( bild, umgebungsModel_dst, CV_GRAY2BGR );

	if( minimumIndex != -1 )
	{
		float* line = (float*)cvGetSeqElem(lines,minimumIndex);
		float rho = line[0];
		float theta = line[1];
		CvPoint pt1, pt2;
		double a = cos(theta), b = sin(theta);
		double x0 = a*rho, y0 = b*rho;
		pt1.x = cvRound(x0 + 1000*(-b));
		pt1.y = cvRound(y0 + 1000*(a));
		pt2.x = cvRound(x0 - 1000*(-b));
		pt2.y = cvRound(y0 - 1000*(a));
		cvLine( umgebung_dst, pt1, pt2, CV_RGB(0,0,255), 1, 8 );
	}
	if( maximumIndex != -1 )
	{
		float* line = (float*)cvGetSeqElem(lines,maximumIndex);
		float rho = line[0];
		float theta = line[1];
		CvPoint pt1, pt2;
		double a = cos(theta), b = sin(theta);
		double x0 = a*rho, y0 = b*rho;
		pt1.x = cvRound(x0 + 1000*(-b));
		pt1.y = cvRound(y0 + 1000*(a));
		pt2.x = cvRound(x0 - 1000*(-b));
		pt2.y = cvRound(y0 - 1000*(a));
		cvLine( umgebung_dst, pt1, pt2, CV_RGB(255,0,255), 1, 8 );
	}
#endif
	double nearMinimumDistance = 0.f;
	int nearMinimumIndex = -1;
	// second step find a line nearly paralell to this line and with nearly the same distance to robot
	for( int i = 0; i < lines->total; ++i )
	{
		if( i == minimumIndex )
			continue;
		float* line = (float*)cvGetSeqElem(lines,i);
		float rho = line[0];
		float theta = line[1];
		// is the line nearley perpendicular to the robot 
		// theta must be between 60 and 120
		if( theta > (1./3.)*CV_PI && theta < (2./3.)*CV_PI )
		{

			// calculate the distance from the robot 
			double d = cos(theta) * (odoMetrieXScaled - rho*cos(theta) )
				+ sin(theta) * (odoMetrieYScaled - rho*sin(theta ) );
			if( fabs(nearMinimumDistance) > fabs(d)-fabs(minimumDistance) || nearMinimumIndex == -1 )
			{
				nearMinimumDistance = fabs(d)-fabs(minimumDistance);
				nearMinimumIndex = i;
			}

		}
	}
#ifdef DEBUGOUTPUT
	if( nearMinimumIndex != -1 )
	{
		float* line = (float*)cvGetSeqElem(lines,nearMinimumIndex);
		float rho = line[0];
		float theta = line[1];
		CvPoint pt1, pt2;
		double a = cos(theta), b = sin(theta);
		double x0 = a*rho, y0 = b*rho;
		pt1.x = cvRound(x0 + 1000*(-b));
		pt1.y = cvRound(y0 + 1000*(a));
		pt2.x = cvRound(x0 - 1000*(-b));
		pt2.y = cvRound(y0 - 1000*(a));
		cvLine( umgebung_dst, pt1, pt2, CV_RGB(0,255,255), 1, 8 );
	}
#endif

	// we know our model very well so when we have to create six points out of the lines we have 
	// we do use this knowlege:
	// for the left and right point we take the two upper lines generate the function value at the border and take the greater one
	if( minimumIndex > -1 && nearMinimumIndex > -1 )
	{
		// left border x = 0;
		float* line = (float*)cvGetSeqElem(lines,minimumIndex);
		float rho = line[0];
		float theta = line[1];
		Point lb1;
		double a = cos(theta), b = sin(theta);
		double x0 = a*rho, y0 = b*rho;
		lb1.x = x0 - b*x0;
		lb1.y = y0 + a*x0;
		line = (float*)cvGetSeqElem(lines,nearMinimumIndex);
		rho = line[0];
		theta = line[1];
		Point lb2;
		a = cos(theta); b = sin(theta);
		x0 = a*rho; y0 = b*rho;
		lb2.x = x0 - b*x0;
		lb2.y = y0 + a*x0;
		if( lb1.y < lb2.y )
		{
			leftBorderPoint.x = lb1.x;
			leftBorderPoint.y = lb1.y;
		}
		else
		{
			leftBorderPoint.x = lb2.x;
			leftBorderPoint.y = lb2.y;
		}
#ifdef DEBUGOUTPUT
		CvPoint p = cvPoint( static_cast<int>(leftBorderPoint.x) , static_cast<int>(leftBorderPoint.y) );
		cvLine( umgebung_dst, p, p , CV_RGB(255,0,0), 3, 8 );
#endif
	}
	if( minimumIndex > -1  && nearMinimumIndex > -1 )
	{
		// right border x = IMAGESIZEX;
		float* line = (float*)cvGetSeqElem(lines,minimumIndex);
		float rho = line[0];
		float theta = line[1];
		Point rb1;
		double a = cos(theta), b = sin(theta);
		double x0 = a*rho, y0 = b*rho;
		rb1.x = x0 + b*(IMAGESIZEX-(x0+1));
		rb1.y = y0 - a*(IMAGESIZEX-(x0+1));
		line = (float*)cvGetSeqElem(lines,nearMinimumIndex);
		rho = line[0];
		theta = line[1];
		Point rb2;
		a = cos(theta); b = sin(theta);
		x0 = a*rho; y0 = b*rho;
		rb2.x = x0 + b*(IMAGESIZEX-(x0+1));
		rb2.y = y0 - a*(IMAGESIZEX-(x0+1));
		if( rb1.y < rb2.y )
		{
			rightBorderPoint.x = rb1.x;
			rightBorderPoint.y = rb1.y;
		}
		else
		{
			rightBorderPoint.x = rb2.x;
			rightBorderPoint.y = rb2.y;
		}
#ifdef DEBUGOUTPUT
		CvPoint p = cvPoint( static_cast<int>(rightBorderPoint.x) , static_cast<int>(rightBorderPoint.y) );
		cvLine( umgebung_dst, p, p, CV_RGB(255,0,0), 3, 8 );
#endif
	}

	if( minimumIndex > -1 && minimumPositiveIndex  > -1 && nearMinimumIndex > -1 ) 
	{
		// leftTop x = IMAGESIZEX;
		float* line = (float*)cvGetSeqElem(lines,minimumIndex);
		float* line2 = (float*)cvGetSeqElem(lines,minimumPositiveIndex);
		float rho = line[0];
		float theta = line[1];
		float rho2 = line2[0];
		float theta2 = line2[1];

		Point sp1;
		double a = cos(theta), b = sin(theta);
		double a2 = cos(theta2), b2 = sin(theta2);
		double x0 = a*rho, y0 = b*rho;
		double x02 = a2*rho2, y02 = b2*rho2;
		double tempX = (rho/b - rho2/b2)/(1./tan(theta)-1./tan(theta2));
		if( fabs(theta) < 1e-5)
			tempX = rho; 
		else if ( fabs(theta2) < 1e-5 )
			tempX = rho2;
		sp1.x = tempX;
		sp1.y = y0 - a*(tempX-x0);

		line = (float*)cvGetSeqElem(lines,nearMinimumIndex);
		rho = line[0];
		theta = line[1];
		Point sp2;
		a = cos(theta); b = sin(theta);
		x0 = a*rho; y0 = b*rho;
		tempX = (rho/b - rho2/b2)/(1./tan(theta)-1./tan(theta2));
		if( fabs(theta) < 1e-5)
			tempX = rho; 
		else if ( fabs(theta2) < 1e-5 )
			tempX = rho2;
		sp2.x = tempX;
		sp2.y = y0 - a*(tempX-x0);
		if( sp1.y < sp2.y )
		{
			leftTopPoint.x = sp1.x;
			leftTopPoint.y = sp1.y;
		}
		else
		{
			leftTopPoint.x = sp2.x;
			leftTopPoint.y = sp2.y;
		}
#ifdef DEBUGOUTPUT
		CvPoint p = cvPoint( static_cast<int>(leftTopPoint.x) , static_cast<int>(leftTopPoint.y) );
		cvLine( umgebung_dst, p, p, CV_RGB(255,0,0), 3, 8 );
#endif
	}
	if( maximumIndex > -1 && minimumPositiveIndex > -1 && nearMinimumIndex > -1 )
	{
		// leftBottom x = IMAGESIZEX;
		float* line = (float*)cvGetSeqElem(lines,maximumIndex);
		float* line2 = (float*)cvGetSeqElem(lines,minimumPositiveIndex);
		float rho = line[0];
		float theta = line[1];
		float rho2 = line2[0];
		float theta2 = line2[1];

		double a = cos(theta), b = sin(theta);
		double a2 = cos(theta2), b2 = sin(theta2);
		double x0 = a*rho, y0 = b*rho;
		double x02 = a2*rho2, y02 = b2*rho2;
		double tempX = (rho/b - rho2/b2)/(1./tan(theta)-1./tan(theta2));
		if( fabs(theta) < 1e-5)
			tempX = rho; 
		else if ( fabs(theta2) < 1e-5 )
			tempX = rho2;
		leftBottomPoint.x = tempX;
		leftBottomPoint.y = y0 - a*(tempX-x0);

#ifdef DEBUGOUTPUT
		CvPoint p = cvPoint( static_cast<int>(leftBottomPoint.x) , static_cast<int>(leftBottomPoint.y) );
		cvLine( umgebung_dst, p, p, CV_RGB(255,0,0), 3, 8 );
#endif
	}

	if( minimumIndex > -1 && minimumNegativeIndex > -1 && nearMinimumIndex > -1 )
	{
		// rightTop x = IMAGESIZEX;
		float* line = (float*)cvGetSeqElem(lines,minimumIndex);
		float* line2 = (float*)cvGetSeqElem(lines,minimumNegativeIndex);
		float rho = line[0];
		float theta = line[1];
		float rho2 = line2[0];
		float theta2 = line2[1];

		Point sp1;
		double a = cos(theta), b = sin(theta);
		double a2 = cos(theta2), b2 = sin(theta2);
		double x0 = a*rho, y0 = b*rho;
		double x02 = a2*rho2, y02 = b2*rho2;
		double tempX = (rho/b - rho2/b2)/(1./tan(theta)-1./tan(theta2));
		if( fabs(theta) < 1e-5)
			tempX = rho; 
		else if ( fabs(theta2) < 1e-5 )
			tempX = rho2;

		sp1.x = tempX;
		sp1.y = y0 - a*(tempX-x0);
		line = (float*)cvGetSeqElem(lines,nearMinimumIndex);
		rho = line[0];
		theta = line[1];
		Point sp2;
		a = cos(theta); b = sin(theta);
		x0 = a*rho; y0 = b*rho;
		tempX = (rho/b - rho2/b2)/(1./tan(theta)-1./tan(theta2));
		if( fabs(theta) < 1e-5)
			tempX = rho; 
		else if ( fabs(theta2) < 1e-5 )
			tempX = rho2;
		//sp2.x = (x02*(1-b2) + x0*(b-1)) /(b-b2);
		//sp2.y = y0 - a*(sp1.x-x0);
		sp2.x = tempX;
		sp2.y = y0 - a*(tempX-x0);
		if( sp1.y < sp2.y )
		{
			rightTopPoint.x = sp1.x;
			rightTopPoint.y = sp1.y;
		}
		else
		{
			rightTopPoint.x = sp2.x;
			rightTopPoint.y = sp2.y;
		}
#ifdef DEBUGOUTPUT
		CvPoint p = cvPoint( static_cast<int>(rightTopPoint.x) , static_cast<int>(rightTopPoint.y) );
		cvLine( umgebung_dst, p, p, CV_RGB(255,0,0), 3, 8 );
#endif
	}
	if( maximumIndex > -1 && minimumNegativeIndex > -1 && nearMinimumIndex > -1 )
	{
		// leftBottom x = IMAGESIZEX;
		float* line = (float*)cvGetSeqElem(lines,maximumIndex);
		float* line2 = (float*)cvGetSeqElem(lines,minimumNegativeIndex);
		float rho = line[0];
		float theta = line[1];
		float rho2 = line2[0];
		float theta2 = line2[1];

		double a = cos(theta), b = sin(theta);
		double a2 = cos(theta2), b2 = sin(theta2);
		double x0 = a*rho, y0 = b*rho;
		double x02 = a2*rho2, y02 = b2*rho2;
		double tempX = (rho/b - rho2/b2)/(1./tan(theta)-1./tan(theta2));
		if( fabs(theta) < 1e-5)
			tempX = rho;
		else if ( fabs(theta2) < 1e-5 )
			tempX = rho2;
		rightBottomPoint.x = tempX;
		rightBottomPoint.y = y0 - a*(tempX-x0);
#ifdef DEBUGOUTPUT
		CvPoint p = cvPoint( static_cast<int>(rightBottomPoint.x) , static_cast<int>(rightBottomPoint.y) );
		cvLine( umgebung_dst, p, p, CV_RGB(255,0,0), 3, 8 );
#endif
	}
#ifdef DEBUGOUTPUT
	cvNamedWindow( "Umgebung", 1 );
	cvShowImage( "Umgebung", umgebung_dst );

	cvNamedWindow( "Umgebungsmodel", 1 );
	cvShowImage( "Umgebungsmodel", umgebungsModel_dst );
#endif

	// do a transformation back in the roboter space
	leftBorderPoint.x = (leftBorderPoint.x + shiftX)/factorX;
	leftBorderPoint.y = (leftBorderPoint.y + shiftY)/factorY;
	rightBorderPoint.x = (rightBorderPoint.x + shiftX)/factorX;
	rightBorderPoint.y = (rightBorderPoint.y + shiftY)/factorY;
	leftTopPoint.x = (leftTopPoint.x + shiftX)/factorX;
	leftTopPoint.y = (leftTopPoint.y + shiftY)/factorY;
	leftBottomPoint.x = (leftBottomPoint.x + shiftX)/factorX;
	leftBottomPoint.y = (leftBottomPoint.y + shiftY)/factorY;
	rightTopPoint.x = (rightTopPoint.x + shiftX)/factorX;
	rightTopPoint.y = (rightTopPoint.y + shiftY)/factorY;
	rightBottomPoint.x = (rightBottomPoint.x + shiftX)/factorX;
	rightBottomPoint.y = (rightBottomPoint.y + shiftY)/factorY;

	// release image
	cvReleaseImage( &bild );
	cvReleaseImage( &color_dst );
	cvReleaseMemStorage( &storage );

#ifdef DEBUGOUTPUT
	cvReleaseImage( &umgebung_dst );
	cvReleaseImage( &umgebungsModel_dst );
#endif

	// do the back transformation from laserscanner coords to world coords
	leftBorderPoint = rotatePoint( leftBorderPoint, odoMetrieTheta );
	leftBorderPoint.x = leftBorderPoint.x + odoMetrieX;
	leftBorderPoint.y = leftBorderPoint.y + odoMetrieY;
	rightBorderPoint = rotatePoint( rightBorderPoint, odoMetrieTheta );
	rightBorderPoint.x = rightBorderPoint.x + odoMetrieX;
	rightBorderPoint.y = rightBorderPoint.y + odoMetrieY;
	leftTopPoint = rotatePoint( leftTopPoint, odoMetrieTheta );
	leftTopPoint.x = leftTopPoint.x + odoMetrieX;
	leftTopPoint.y = leftTopPoint.y + odoMetrieY;
	leftBottomPoint = rotatePoint( leftBottomPoint, odoMetrieTheta );
	leftBottomPoint.x = leftBottomPoint.x + odoMetrieX;
	leftBottomPoint.y = leftBottomPoint.y + odoMetrieY;
	rightTopPoint = rotatePoint( rightTopPoint, odoMetrieTheta );
	rightTopPoint.x = rightTopPoint.x + odoMetrieX;
	rightTopPoint.y = rightTopPoint.y + odoMetrieY;
	rightBottomPoint = rotatePoint( rightBottomPoint, odoMetrieTheta );
	rightBottomPoint.x = rightBottomPoint.x + odoMetrieX;
	rightBottomPoint.y = rightBottomPoint.y + odoMetrieY;

	return (minimumIndex != -1 && maximumIndex != -1 && nearMinimumIndex != -1 && minimumPositiveIndex != -1 && minimumNegativeIndex );
}

Position mp_of_ps(Point* lt, Point* rt, Point* lb, Point* rb) {
    Point coord;
    
    // compute the coord of midpoint
    double num =    lb->x*rt->y + rt->x*rb->y + rb->x*lb->y - 
		    lb->x*rb->y - rb->x*rt->y - rt->x*lb->y;

    double denum =  lt->x*rt->y + rb->x*lb->y + rt->x*rb->y + lb->x*lt->y - 
		    rb->x*rt->y - lt->x*lb->y - rt->x*lt->y - lb->x*rb->y;
    
    double t = num / denum;

    coord.x = rb->x + t*(lt->x - rb->x);
    coord.y = rb->y + t*(lt->y - rb->y);
    
    // compute theta-angle
    double y = rb->y - lb->y;
    if(y == 0)	// the line (rb -lb) is collateral to x-axis
                // => theta = 0!
        return Position(coord.x, coord.y, 0.0);

	else {
	    double x = rb->x- lb->x;
		double theta = acos(x / sqrt(x*x + y*y));
		
		return Position(coord.x, coord.y, theta);
	}
}

Point rotatePoint( const Point &point, float angle )
{
	Point retVal;
	retVal.x = point.x * cos(angle) - point.y * sin(angle);
	retVal.y = point.x * sin(angle) + point.y * cos(angle);
	return retVal;
}
