#ifndef FINDPARKINGLOT_H
#define FINDPARKINGLOT_H

#define IMAGESIZEX 256
#define IMAGESIZEY 64

#include "Point.h"
#include "Position.h"

#include <vector>

// this function tries to find a parking lot in a laserscan
// laserScanPoints - a array of double values, representing the laserscan values
// laserScanLenght - the number of laserscan points
// pOdoMetrie - a pointer to the Position-Object, representing the postion of the robot, must not be NULL
// leftBorderPoint - this point will be set with the value of the left point of the parking lot, if function succeed otherwise it's undefined
// leftTopPoint - this point will be set with the value of the left top point of the parking lot, if function succeed otherwise it's undefined
// leftBottomPoint - this point will be set with the value of the left bottom point of the parking lot, if function succeed otherwise it's undefined
// rightBottomPoint - this point will be set with the value of the right bottom point of the parking lot, if function succeed otherwise it's undefined
// rightTopPoint - this point will be set with the value of the right top point of the parking lot, if function succeed otherwise it's undefined
// rightBorderPoint - this point will be set with the value of the right point of the parking lot, if function succeed otherwise it's undefined
bool findParkingLot( double* laserScanPoints, 
					size_t laserScanLength,
					Position* pOdoMetrie,
					Point &leftBorderPoint, 
					Point &leftTopPoint, 
					Point &leftBottomPoint,
					Point &rightBottomPoint,
					Point &rightTopPoint, 
					Point &rightBorderPoint );

/** Compute the midpoint and the theta angle of target point
  * @return target position
  */
Position mp_of_ps(Point* lt, Point* rt, Point* lb, Point* rb);

/** Compute the rotation of a point with an given angle
  * @return rotated postion
  */
Point rotatePoint( const Point &point, float angle );

#endif //FINDPARKINGLOT_H
