/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * Master of Science - Computergraphics and Image Processing
 *
 * Robotics - Prof. Dr. Max Fischer
 * "Autonomous self-parking robot"
 * 
 * System: Linux, C++
 */

/**
 * USABILITY:
 *  Copyright (c) 2006 
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Class Description:
 *	Global constants
 *
 * @author Thomas Wöllert, Beatrix Müller, Herbert Hundsnurscher
 */

#ifndef GLOBAL_H
#define GLOBAL_H

#include <string>
#include <sstream>

#include "Robot.h"
#include "Environment.h"
#include "CVCoordDrawingArea.h"

// The main robot instance
Robot* g_Robot;

// The main environment instance
Environment* g_Environment;

// The main coordinate drawing area
CVCoordDrawingArea* g_CoordDrawingArea;

/**
* Transform the given object (i.e. int, double, etc.) into a string
* @param T& the object to transform
* @return the string
*/
template<class T> string TtoString(const T& _t) {
	ostringstream ost;
	ost << _t;
	return ost.str();
}

/**
* Transform the given string into an object (i.e. int, double, etc.)
* @param string& the string to transform
* @return the object
*/
template<class T> T stringToT(const string& _s) {
	T t;
	istringstream ist(_s);
	ist >> t;
	return t;
}

#endif // GLOBAL_H
