#include "Joystick.h"
#include <stdio.h>
#include <string.h>

#ifdef WITH_LINUX_JS
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/select.h>

#include <linux/joystick.h>
#endif

using namespace std;

Joystick::Joystick(string dev) {
	joystickFound = false;
	was_pressed = false;
	had_button_event = false;
	pressedButton = -1;
#ifdef WITH_LINUX_JS
	fd = open(dev.c_str(), O_RDONLY);
	if(fd == -1) {
		fprintf(stderr, "could not open js-device '%s': %s\n", dev.c_str(), strerror(errno));
		//abort();
	} else {
		joystickFound = true;
	}
#else
	printf("no joystick support\n");
#endif
}

bool Joystick::joystickSupported() {
	return joystickFound;
}

bool Joystick::wait_event(unsigned int ms) {
#ifdef WITH_LINUX_JS
	bool had_events = false;
	fd_set rfd;
	struct js_event ev;
	struct timeval tv = {0, ms * 1000};
	
	while(1) {
		FD_SET(fd, &rfd);
		int n = select(fd + 1, &rfd, NULL, NULL, &tv);
		if(n == -1) {
			fprintf(stderr, "js-select()-error: %s\n", strerror(errno));
			abort();
		}
		if(n == 0)
			return had_events;  // timeout
		
		int count = read(fd, &ev, sizeof(ev));
		if(count != sizeof(ev)) {
			fprintf(stderr, "js-read()-error: %s\n", strerror(errno));
			abort();
		}
	
		// printf("js:  time:%15d, value:%5d, type: %5d, number: %5d\n", ev.time, ev.value, ev.type, ev.number);
	
		if(ev.type == JS_EVENT_BUTTON) {
			//printf("Joystick button number %d : value = %d\n", ev.number, ev.value);
			bool pressed = false;
			pressedButton = ev.number;
			pressed = ev.value;
			//had_button_event = pressed && !was_pressed;
			had_button_event = pressed;
			was_pressed = pressed;
			had_events = true;
		}
		
		if(!ev.type & JS_EVENT_AXIS)
			continue;
		
		if(ev.number == JS_THROTTLE_AXIS) {
			throttle = (double)ev.value / 32768.0;
			had_events = true;
		} else if (ev.number == JS_STEERING_AXIS) {
			steering = (double)ev.value / 32768.0;
			had_events = true;
		}
	}
#else
	usleep(ms * 1000);
#endif
}
