/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * Master of Science - Computergraphics and Image Processing
 *
 * Robotics - Prof. Dr. Max Fischer
 * "Autonomous self-parking robot"
 * 
 * System: Linux, C++
 */

/**
 * USABILITY:
 *  Copyright (c) 2006 
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Class Description:
 *	User Planer
 *
 * @author Thomas Wöllert
 */

#ifndef USERPLANER_H
#define USERPLANER_H

#include "Joystick.h"
#include "AStarPlaner.h"

#include <iostream>
#include <vector>

// Key defines
#define KEY_MOVE_FORWARD_STRAIGHT	's'
#define KEY_MOVE_FORWARD_LEFT		'a'
#define KEY_MOVE_FORWARD_RIGHT		'd'
#define KEY_MOVE_STOP			'f'

#define KEY_MOVE_BACKWARD_STRAIGHT	'x'
#define KEY_MOVE_BACKWARD_LEFT		'c'
#define KEY_MOVE_BACKWARD_RIGHT		'y'

#define KEY_START_PARKING		'p'

#define KEY_DRIVE_REVERSE		'r'

#define INTERPOLATE_LASER_DATA

class Car;

using namespace std;

struct laserdata {
	int count;
	double* data;
};

class UserPlaner {
	protected:
		// The car model used in this planer
		Car* car;
        
		// The joystick
        	Joystick js;
		
		// The time step to use
		double deltaT;
		
		// The AStarPlaner
		AStarPlaner* starPlaner;
	
	public:
		/**
		* Constructor
		* @param Car* the car model to use
		* @param double the time step to use
		*/
		UserPlaner(Car* _car, double _deltaT);
	
		/**
		* Destructor
		*/
		virtual ~UserPlaner();
	
		/**
		* Start the algorithm
		*/
		virtual void run();
		
	protected:
		int waitForParkingKey();
		void initParking();
		laserdata interpolateLaserData(double* _dataArray, int _dataCount);
};

#endif // USERPLANER_H
