/*
	kollisionstest fuer beliebige recht- und dereiecke
	v 0.2
	
	von Khan Hoang van
	und Florian Schmidt

	FHM Muechen
	Tue, 25 Apr 2006 15:37:33 +0200
	
	based on Triangle/triangle intersection test routine,
	by Tomas Moller, 1997.
*/

/* Triangle/triangle intersection test routine,
 * by Tomas Moller, 1997.
 * See article "A Fast Triangle-Triangle Intersection Test",
 * Journal of Graphics Tools, 2(2), 1997
 * updated: 2001-06-20 (added line of intersection)
 *
 * int <a href="#TRITRI">tri_tri_intersect</a>(float V0[3],float V1[3],float V2[3],
 *                       float U0[3],float U1[3],float U2[3])
 *
 * parameters: vertices of triangle 1: V0,V1,V2
 *             vertices of triangle 2: U0,U1,U2
 * result    : returns 1 if the triangles intersect, otherwise 0
 *
 * Here is a version withouts divisions (a little faster)
 * int <a href="#NODIV">NoDivTriTriIsect</a>(float V0[3],float V1[3],float V2[3],
 *                      float U0[3],float U1[3],float U2[3]);
 * 
 * This version computes the line of intersection as well (if they are not coplanar):
 * int <a href="#ISECTLINE">tri_tri_intersect_with_isectline</a>(float V0[3],float V1[3],float V2[3], 
 *				        float U0[3],float U1[3],float U2[3],int *coplanar,
 *				        float isectpt1[3],float isectpt2[3]);
 * coplanar returns whether the tris are coplanar
 * isectpt1, isectpt2 are the endpoints of the line of intersection
 */
 
 /* Date: 2006-04-24
  * Bemerkung: Es werden hier notwaendige Modifikationen, bzw. Funktionen
  * 		   ergaenzt, um Kollisionen-Test zwischen Rechtecken durchfuehren 
  * 		   zu koennen!
  * 	
  * 		1. Ergaenzung: EDGE_AGAINST_RECT_EDGES
  * 		2. Ergaenzung: POINT_IN_RECT
  * 		3. Ergaenzung: int coplanar_rectrect(float N[3],float V0[3],
  * 						float V1[3],float V2[3],
  * 						float U0[3],float U1[3],
  * 						float U2[3])
  */

#include <math.h>
#include <stdio.h>
#include "collision_test.h"

#define FABS(x) ((float)fabs(x))        /* implement as is fastest on your machine */

/* if USE_EPSILON_TEST is true then we do a check: 
         if |dv|<EPSILON then dv=0.0;
   else no check is done (which is less robust)
*/

#define USE_EPSILON_TEST TRUE  
#define EPSILON 0.000001


/* some macros */
#define CROSS(dest,v1,v2)                      \
              dest[0]=v1[1]*v2[2]-v1[2]*v2[1]; \
              dest[1]=v1[2]*v2[0]-v1[0]*v2[2]; \
              dest[2]=v1[0]*v2[1]-v1[1]*v2[0];

#define DOT(v1,v2) (v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2])

#define SUB(dest,v1,v2) dest[0]=v1[0]-v2[0]; dest[1]=v1[1]-v2[1]; dest[2]=v1[2]-v2[2]; 

#define ADD(dest,v1,v2) dest[0]=v1[0]+v2[0]; dest[1]=v1[1]+v2[1]; dest[2]=v1[2]+v2[2]; 

#define MULT(dest,v,factor) dest[0]=factor*v[0]; dest[1]=factor*v[1]; dest[2]=factor*v[2];

#define SET(dest,src) dest[0]=src[0]; dest[1]=src[1]; dest[2]=src[2]; 

/* sort so that a<=b */
#define SORT(a,b)       \
             if(a>b)    \
             {          \
               float c; \
               c=a;     \
               a=b;     \
               b=c;     \
             }

#define ISECT(VV0,VV1,VV2,D0,D1,D2,isect0,isect1) \
              isect0=VV0+(VV1-VV0)*D0/(D0-D1);    \
              isect1=VV0+(VV2-VV0)*D0/(D0-D2);


#define COMPUTE_INTERVALS(VV0,VV1,VV2,D0,D1,D2,D0D1,D0D2,isect0,isect1) \
  if(D0D1>0.0f)                                         \
  {                                                     \
    /* here we know that D0D2<=0.0 */                   \
    /* that is D0, D1 are on the same side, D2 on the other or on the plane */ \
    ISECT(VV2,VV0,VV1,D2,D0,D1,isect0,isect1);          \
  }                                                     \
  else if(D0D2>0.0f)                                    \
  {                                                     \
    /* here we know that d0d1<=0.0 */                   \
    ISECT(VV1,VV0,VV2,D1,D0,D2,isect0,isect1);          \
  }                                                     \
  else if(D1*D2>0.0f || D0!=0.0f)                       \
  {                                                     \
    /* here we know that d0d1<=0.0 or that D0!=0.0 */   \
    ISECT(VV0,VV1,VV2,D0,D1,D2,isect0,isect1);          \
  }                                                     \
  else if(D1!=0.0f)                                     \
  {                                                     \
    ISECT(VV1,VV0,VV2,D1,D0,D2,isect0,isect1);          \
  }                                                     \
  else if(D2!=0.0f)                                     \
  {                                                     \
    ISECT(VV2,VV0,VV1,D2,D0,D1,isect0,isect1);          \
  }                                                     \
  else                                                  \
  {                                                     \
    /* triangles are coplanar */                        \
    return coplanar_tri_tri(N1,V0,V1,V2,U0,U1,U2);      \
  }



/* this edge to edge test is based on Franlin Antonio's gem:
   "Faster Line Segment Intersection", in Graphics Gems III,
   pp. 199-202 */
#define EDGE_EDGE_TEST(V0,U0,U1)                      \
  Bx=U0[i0]-U1[i0];                                   \
  By=U0[i1]-U1[i1];                                   \
  Cx=V0[i0]-U0[i0];                                   \
  Cy=V0[i1]-U0[i1];                                   \
  f=Ay*Bx-Ax*By;                                      \
  d=By*Cx-Bx*Cy;                                      \
  if((f>0 && d>=0 && d<=f) || (f<0 && d<=0 && d>=f))  \
  {                                                   \
    e=Ax*Cy-Ay*Cx;                                    \
    if(f>0)                                           \
    {                                                 \
      if(e>=0 && e<=f) return 1;                      \
    }                                                 \
    else                                              \
    {                                                 \
      if(e<=0 && e>=f) return 1;                      \
    }                                                 \
  }                                

#define EDGE_AGAINST_TRI_EDGES(V0,V1,U0,U1,U2) \
{                                              \
  float Ax,Ay,Bx,By,Cx,Cy,e,d,f;               \
  Ax=V1[i0]-V0[i0];                            \
  Ay=V1[i1]-V0[i1];                            \
  /* test edge U0,U1 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U0,U1);                    \
  /* test edge U1,U2 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U1,U2);                    \
  /* test edge U2,U1 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U2,U0);                    \
}

/* 1. Modifikation:						 
 * 	Ueberpruefe, ob eine Kante von 1. Rechteck sich mit der	
 * 	Kanten des anderen Rechtecks ueberschneidet?		
 */								
#define EDGE_AGAINST_RECT_EDGES(V0,V1,U0,U1,U2, U3) \
{                                              \
  float Ax,Ay,Bx,By,Cx,Cy,e,d,f;               \
  Ax=V1[i0]-V0[i0];                            \
  Ay=V1[i1]-V0[i1];                            \
  /* test edge U0,U1 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U0,U1);                    \
  /* test edge U1,U2 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U1,U2);                    \
  /* test edge U2,U3 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U2,U3);                    \
  /* test edge U3,U0 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U3,U0);                    \
}


#define POINT_IN_TRI(V0,U0,U1,U2)           \
{                                           \
  float a,b,c,d0,d1,d2;                     \
  /* is T1 completly inside T2? */          \
  /* check if V0 is inside tri(U0,U1,U2) */ \
  a=U1[i1]-U0[i1];                          \
  b=-(U1[i0]-U0[i0]);                       \
  c=-a*U0[i0]-b*U0[i1];                     \
  d0=a*V0[i0]+b*V0[i1]+c;                   \
                                            \
  a=U2[i1]-U1[i1];                          \
  b=-(U2[i0]-U1[i0]);                       \
  c=-a*U1[i0]-b*U1[i1];                     \
  d1=a*V0[i0]+b*V0[i1]+c;                   \
                                            \
  a=U0[i1]-U2[i1];                          \
  b=-(U0[i0]-U2[i0]);                       \
  c=-a*U2[i0]-b*U2[i1];                     \
  d2=a*V0[i0]+b*V0[i1]+c;                   \
  if(d0*d1>0.0)                             \
  {                                         \
    if(d0*d2>0.0) return 1;                 \
  }                                         \
}

/* 2. Modifikatiion: 							
 * 	Testen, ob ein Punkt sich innerhalb eines Rechtecks befindet? 	
 */									
#define POINT_IN_RECT(V0,U0,U1,U2, U3)      \
{                                           \
  float a,b,c,d0,d1,d2,d3;                     \
  /* is rect1 completly inside rect2? */          \
  /* check if V0 is inside rect(U0,U1,U2,U3) */ \
  a=U1[i1]-U0[i1];                          \
  b=-(U1[i0]-U0[i0]);                       \
  c=-a*U0[i0]-b*U0[i1];                     \
  d0=a*V0[i0]+b*V0[i1]+c;                   \
                                            \
  a=U2[i1]-U1[i1];                          \
  b=-(U2[i0]-U1[i0]);                       \
  c=-a*U1[i0]-b*U1[i1];                     \
  d1=a*V0[i0]+b*V0[i1]+c;                   \
                                            \
  a=U3[i1]-U2[i1];                          \
  b=-(U3[i0]-U2[i0]);                       \
  c=-a*U2[i0]-b*U2[i1];                     \
  d2=a*V0[i0]+b*V0[i1]+c;                   \
                                            \
  a=U0[i1]-U3[i1];                          \
  b=-(U0[i0]-U2[i0]);                       \
  c=-a*U3[i0]-b*U3[i1];                     \
  d3=a*V0[i0]+b*V0[i1]+c;                   \
					    \
  if(d0*d1>0.0)                             \
  {                                         \
    if(d1*d2>0.0)                           \
    {					    \
        if(d2*d3 > 0.0) return 1;           \
    }					    \
  }                                         \
}

int coplanar_tri_tri(float N[3],float V0[3],float V1[3],float V2[3],
                     float U0[3],float U1[3],float U2[3])
{
   float A[3];
   short i0,i1;
   /* first project onto an axis-aligned plane, that maximizes the area */
   /* of the triangles, compute indices: i0,i1. */
   A[0]=fabs(N[0]);
   A[1]=fabs(N[1]);
   A[2]=fabs(N[2]);
   if(A[0]>A[1])
   {
      if(A[0]>A[2])  
      {
          i0=1;      /* A[0] is greatest */
          i1=2;
      }
      else
      {
          i0=0;      /* A[2] is greatest */
          i1=1;
      }
   }
   else   /* A[0]<=A[1] */
   {
      if(A[2]>A[1])
      {
          i0=0;      /* A[2] is greatest */
          i1=1;                                           
      }
      else
      {
          i0=0;      /* A[1] is greatest */
          i1=2;
      }
    }               
                
    /* test all edges of triangle 1 against the edges of triangle 2 */
    EDGE_AGAINST_TRI_EDGES(V0,V1,U0,U1,U2);
    EDGE_AGAINST_TRI_EDGES(V1,V2,U0,U1,U2);
    EDGE_AGAINST_TRI_EDGES(V2,V0,U0,U1,U2);
                
    /* finally, test if tri1 is totally contained in tri2 or vice versa */
    POINT_IN_TRI(V0,U0,U1,U2);
    POINT_IN_TRI(U0,V0,V1,V2);

    return 0;
}

/* 3. Modifikation: 
 * 	Testen, ob 2 gegebenen Rechtecken sich ueberschneidet?
 * 	Hier wird immer davon ausgeganen, dass die zwei gegebene 
 * 	Rechtecken auf einen gemeinsamen Ebene liegen!
 */
int coplanar_rectrect(float N[3],float V0[3],float V1[3],float V2[3],float V3[3],
                     float U0[3],float U1[3],float U2[3],float U3[3])
{
   float A[3];
   short i0,i1;
   /* first project onto an axis-aligned plane, that maximizes the area */
   /* of the triangles, compute indices: i0,i1. */
   A[0]=fabs(N[0]);
   A[1]=fabs(N[1]);
   A[2]=fabs(N[2]);
   if(A[0]>A[1])
   {
      if(A[0]>A[2])  
      {
          i0=1;      /* A[0] is greatest */
          i1=2;
      }
      else
      {
          i0=0;      /* A[2] is greatest */
          i1=1;
      }
   }
   else   /* A[0]<=A[1] */
   {
      if(A[2]>A[1])
      {
          i0=0;      /* A[2] is greatest */
          i1=1;                                           
      }
      else
      {
          i0=0;      /* A[1] is greatest */
          i1=2;
      }
    }               
                
    /* test all edges of rectanle 1 against the edges of rectangle 2 */
    EDGE_AGAINST_RECT_EDGES(V0,V1,U0,U1,U2,U3);
    EDGE_AGAINST_RECT_EDGES(V1,V2,U0,U1,U2,U3);
    EDGE_AGAINST_RECT_EDGES(V2,V3,U0,U1,U2,U3);
    EDGE_AGAINST_RECT_EDGES(V3,V0,U0,U1,U2,U3);
                
    /* finally, test if rect1 is totally contained in rect2 or vice versa */
    POINT_IN_RECT(V0,U0,U1,U2,U3);
    POINT_IN_RECT(U0,V0,V1,V2,V3);

    return 0;
}


int rectrect_collision(CvPoint* rect, CvPoint* others, int others_count, bool* collisions) {
	int n = 0;
	float v10[3] = {rect[0].x, rect[0].y, 0};
	float v11[3] = {rect[1].x, rect[1].y, 0};
	float v12[3] = {rect[2].x, rect[2].y, 0};
	float v13[3] = {rect[3].x, rect[3].y, 0};
	float E1[3],E2[3], N1[3];
	SUB(E1, v11, v10);
	SUB(E2, v12, v10);
	CROSS(N1, E1, E2);
	for(int i = 0; i < others_count; i++) {
		int po = 4 * i;
		float v20[3] = {others[po + 0].x, others[po + 0].y, 0};
		float v21[3] = {others[po + 1].x, others[po + 1].y, 0};
		float v22[3] = {others[po + 2].x, others[po + 2].y, 0};
		float v23[3] = {others[po + 3].x, others[po + 3].y, 0};
		bool has_collision = coplanar_rectrect(N1, v10, v11, v12, v13, v20, v21, v22, v23);
		if(collisions)
			collisions[i] = has_collision;
		if(has_collision) {
			if(!collisions) // bei zu erst gefundener collision abbrechen
				return 1;
			n++;
		}
	}
	return n;
}

int tritri_collision(CvPoint* triangle, CvPoint* others, int others_count, bool* collisions) {
	int n = 0;
	float v10[3] = {triangle[0].x, triangle[0].y, 0};
	float v11[3] = {triangle[1].x, triangle[1].y, 0};
	float v12[3] = {triangle[2].x, triangle[2].y, 0};
	float E1[3], E2[3], N1[3];
	SUB(E1, v11, v10);
	SUB(E2, v12, v10);
	CROSS(N1, E1, E2);
	for(int i = 0; i < others_count; i++) {
		int po = 3 * i;
		float v20[3] = {others[po + 0].x, others[po + 0].y, 0};
		float v21[3] = {others[po + 1].x, others[po + 1].y, 0};
		float v22[3] = {others[po + 2].x, others[po + 2].y, 0};
		bool has_collision = coplanar_tri_tri(N1, v10, v11, v12, v20, v21, v22);
		if(collisions)
			collisions[i] = has_collision;
		if(has_collision) {
			if(!collisions) // bei zu erst gefundener collision abbrechen
				return 1;
			n++;
		}
	}
	return n;
}

#define EDGE_EDGE_TEST_2D_CV(V0, U0, U1)                            \
  Bx = U0.x - U1.x;                                                 \
  By = U0.y - U1.y;                                                 \
  Cx = V0.x - U0.x;                                                 \
  Cy = V0.y - U0.y;                                                 \
  f = Ay * Bx - Ax * By;                                            \
  d = By * Cx - Bx * Cy;                                            \
  if((f > 0 && d >= 0 && d <= f) || (f < 0 && d <= 0 && d >= f)) {  \
    e = Ax * Cy - Ay * Cx;                                          \
    if(f > 0) {                                                     \
      if(e >= 0 && e <= f) break;                                   \
    } else if(e <= 0 && e >= f) break;                              \
  }

int rectline_collision(Point* rect, Point* others, int others_count, bool* collisions) {
	int n = 0;
	for(int i = 0; i < others_count - 1; i++) {
		int po = i;
		float Ax, Ay, Bx, By, Cx, Cy, e, d, f;
		Ax = others[po + 1].x - others[po].x;
		Ay = others[po + 1].y - others[po].y;
		
		bool has_collision = true;
		while(1) {
			EDGE_EDGE_TEST_2D_CV(others[po], rect[0], rect[1]);
			EDGE_EDGE_TEST_2D_CV(others[po], rect[1], rect[2]);
			EDGE_EDGE_TEST_2D_CV(others[po], rect[2], rect[3]);
			EDGE_EDGE_TEST_2D_CV(others[po], rect[3], rect[0]);
			has_collision = false;
			break;
		}
		
		if(collisions)
			collisions[i] = has_collision;
		if(has_collision) {
			if(!collisions) // bei zu erst gefundener collision abbrechen
				return 1;
			n++;
		}
	}
	return n;
}

// test prozedur mit make test
#ifdef TEST
#include <time.h>

#ifdef HAVE_GUI
#include "highgui.h"
// test programm mit grafischer ausgabe
#define GUI_TEST_LINE 
//#define GUI_TEST_TRI
//#define GUI_TEST_RECT

#define cycles_per_second 1536973000
#define rdtsc(X) asm volatile("rdtsc" : "=A" (X))
typedef unsigned long long ull;

int main(int argc, char **argv)
{
	// int zoom = 32;
	int zoom = 1; // 4
	int is = 900;
	int (*check_collision)(CvPoint*, CvPoint*, int, bool*) = NULL;
	
#ifdef GUI_TEST_TRI
	check_collision = tritri_collision;
	int per_element = 3;
#else 
#ifdef GUI_TEST_LINE
	check_collision = rectline_collision;
	int per_element = 2;
#else
	check_collision = rectrect_collision;
	int per_element = 4;
#endif
#endif
	
	IplImage *img = cvCreateImage(cvSize(is, is), 8, 3);
	cvNamedWindow("gui_test", 1);
	while(1) {
		int count = 50; // 15;
		bool * collusions = (bool *) malloc((count - 1) * sizeof(bool));
#ifdef GUI_TEST_LINE
		CvPoint *points = (CvPoint *) malloc((4 + per_element * (count - 1)) * sizeof(CvPoint));
		{
			// construct rectangle
			CvPoint* p = &points[0];
			int xvar = img->width / (2 * zoom);
			int yvar = img->height / (2 * zoom);
			p[0].x = rand() % xvar + xvar / 2;
			p[0].y = rand() % yvar + yvar / 2;
			p[2].x = p[0].x + rand() % xvar / 2 - xvar / 4;
			p[2].y = p[0].y + rand() % yvar / 2 - yvar / 4;
			double dx = p[2].x - p[0].x;
			double dy = p[2].y - p[0].y;
			p[1].x = p[0].x + dx / 2.0 - dy / 2.0;
			p[1].y = p[0].y + dy / 2.0 + dx / 2.0;
			p[3].x = p[0].x + dx / 2.0 + dy / 2.0;
			p[3].y = p[0].y + dy / 2.0 - dx / 2.0;
		}
		for(int t = 0; t < count - 1; t++) {
			CvPoint* p = &points[4 + per_element * t];
			int xvar = img->width / (2 * zoom);
			int yvar = img->height / (2 * zoom);
			p[0].x = rand() % xvar + xvar / 2;
			p[0].y = rand() % yvar + yvar / 2;
			for(int op = 1; op < per_element; op++) {
				p[op].x = p[0].x + rand() % xvar / 2 - xvar / 4;
				p[op].y = p[0].y + rand() % yvar / 2 - yvar / 4;
			}
		}
#else
		CvPoint *points = (CvPoint *) malloc(per_element * count * sizeof(CvPoint));
		for(int t = 0; t < count; t++) {
			CvPoint* p = &points[per_element * t];
			int xvar = img->width / (2 * zoom);
			int yvar = img->height / (2 * zoom);
			p[0].x = rand() % xvar + xvar / 2;
			p[0].y = rand() % yvar + yvar / 2;
			for(int op = 1; op < per_element; op++) {
				p[op].x = p[0].x + rand() % xvar / 2 - xvar / 4;
				p[op].y = p[0].y + rand() % yvar / 2 - yvar / 4;
			}
		}
#endif
		
#ifdef GUI_TEST_TRI
		points[per_element + 0].x = 5 / zoom; points[per_element + 0].y = 5 / zoom; 
		points[per_element + 1].x = (img->width - 5) / zoom; points[per_element + 1].y = 5 / zoom; 
		points[per_element + 2].x = (img->width - 5) / zoom; points[per_element + 2].y = (img->height - 5) / zoom; 
#endif
		
		ull start;
		rdtsc(start);
#ifdef GUI_TEST_LINE
		int n = check_collision(&points[0], &points[4], count - 1, collusions);
#else
		int n = check_collision(&points[0], &points[per_element], count - 1, collusions);
#endif		
		ull stop;
		rdtsc(stop);
		
		ull duration = stop - start;
		printf("cycles: %7lld   %8.3fus   %8.0f tests/s gegen %d objecte   %8.0f tests/s gegen ein object   %8.0f tests/s gegen 6 objecte\n", 
			duration,
			duration * 1000000.0 / (double) cycles_per_second,
			1.0 / ((double)duration / (double) cycles_per_second), count - 1,
			1.0 / (((double)duration / (double)(count - 1)) / (double)cycles_per_second),
			1.0 / (((double)duration / (double)(count - 1) * 6) / (double)cycles_per_second)
		);
		
		cvZero(img);
#ifndef GUI_TEST_LINE
		for(int t = 0; t < count; t++) {
			CvScalar color;
			if(t == 0)
				color = CV_RGB(255, 255, 0); // our car
			else if(collusions[t - 1]) {
				color = CV_RGB(255, 0, 0); // collusion
			} else
				color = CV_RGB(0, 255, 0); // no collusion
			
			CvPoint pt0 = points[t * per_element + (per_element - 1)];
			pt0.x *= zoom;
			pt0.y *= zoom;
			for(int i = 0; i < per_element; i++) {
				CvPoint pt = points[t * per_element + i];
				pt.x *= zoom;
				pt.y *= zoom;
				cvLine(img, pt0, pt, color, 1, CV_AA, 0);
				pt0 = pt;
			}
		}
#else
		// first draw rectangle
		CvScalar color = CV_RGB(255, 255, 0); // our car
		per_element = 4;
		CvPoint pt0 = points[per_element - 1];
		pt0.x *= zoom;
		pt0.y *= zoom;
		for(int i = 0; i < per_element; i++) {
			CvPoint pt = points[i];
			pt.x *= zoom;
			pt.y *= zoom;
			cvLine(img, pt0, pt, color, 1, CV_AA, 0);
			pt0 = pt;
		}
		
		// then lines
		per_element = 2;
		for(int t = 0; t < count - 1; t++) {
			if(collusions[t]) {
				color = CV_RGB(255, 0, 0); // collusion
			} else
				color = CV_RGB(0, 255, 0); // no collusion
			
			CvPoint pt0 = points[4 + t * per_element + (per_element - 1)];
			pt0.x *= zoom;
			pt0.y *= zoom;
			for(int i = 0; i < per_element; i++) {
				CvPoint pt = points[4 + t * per_element + i];
				pt.x *= zoom;
				pt.y *= zoom;
				cvLine(img, pt0, pt, color, 1, CV_AA, 0);
				pt0 = pt;
			}
		}
		
#endif
		cvShowImage("gui_test", img);
		int key = cvWaitKey(0);
		if(key == 27 || key == 'q' || key == 'Q')
			break;
		free(points);
		free(collusions);
	}
	cvDestroyWindow("gui_test");
	return 0;
}
#else
// testprogramm ohne grafische ausgabe
int main(int argc, char* argv[]) {
	int count = 10; // anzahl der objekte
	bool* collisions = (bool *) malloc((count) * sizeof(bool)); // wenn collusion[0] == true, dann gibt es eine ueberschneidung mit dem ersten anderen objekt
	int n; // anzahl der erkannten collisionen

	int xvar = 100; // alle x/y koordinaten jedes punktes liegen zwischen 0..100
	int yvar = xvar;
	
	srand(time(NULL));
	
	// test rectangles
	CvPoint *points = (CvPoint *) malloc(4 * count * sizeof(CvPoint));
	for(int t = 0; t < count; t++) { // 10 zufaellige 4ecke generieren
		CvPoint* p = &points[4 * t];
		p[0].x = rand() % xvar;
		p[0].y = rand() % yvar;
		for(int op = 1; op < 4; op++) { // 3 andere punkte in der naehe des ersten punktes
			p[op].x = p[0].x + rand() % xvar / 2;
			p[op].y = p[0].y + rand() % yvar / 2;
		}
	}
	n = rectrect_collision(&points[0] /* das erste rechteck */, &points[4], count - 1 /* anzahl der anderen rechtecke */, collisions);
	printf("gefundene rechteck collisionen: %d\n", n);
	free(points);
	
	// test rectangles
	points = (CvPoint *) malloc(3 * count * sizeof(CvPoint));
	for(int t = 0; t < count; t++) { // 10 zufaellige 3ecke generieren
		CvPoint* p = &points[3 * t];
		p[0].x = rand() % xvar;
		p[0].y = rand() % yvar;
		for(int op = 1; op < 3; op++) { // 2 andere punkte in der naehe des ersten punktes
			p[op].x = p[0].x + rand() % xvar / 2;
			p[op].y = p[0].y + rand() % yvar / 2;
		}
	}
	n = tritri_collision(&points[0] /* das erste dreieck */, &points[3], count - 1 /* anzahl der anderen dreiecke */, collisions);
	printf("gefundene dreieck collisionen1: %d\n", n);
	free(points);
	
	n = tritri_collision(&points[0] /* das erste dreieck */, &points[3], count - 1 /* anzahl der anderen dreiecke */);
	printf("wurde eine dreieck collision gefunden: %d\n", n);
	
	free(collisions);
	return 0;
}
#endif // HIGHGUI
#endif
